@@ -113,6 +113,28 @@ out:
 
 #else
 
+#ifndef HAVE_STRNDUP
+static char
+*strndup ( const char *s, size_t n)
+{
+        size_t nAvail;
+        char *p;
+
+        if ( !s )
+                return NULL;
+
+        if ( strlen(s) > n )
+                nAvail = n + 1;
+        else
+                nAvail = strlen(s) + 1;
+        p = malloc ( nAvail );
+        memcpy ( p, s, nAvail );
+        p[nAvail - 1] = '\0';
+
+        return p;
+}
+#endif
+
 char *
 kit_strdup (const char *s)
 {
@@ -127,6 +149,26 @@ kit_strndup (const char *s, size_t n)
 
 #endif /* KIT_BUILD_TESTS */
 
+#ifdef HAVE_SOLARIS
+int vasprintf(char **strp, const char *fmt, va_list ap)
+{
+        int size;
+        va_list ap2;
+        char s;
+
+        *strp = NULL;
+        va_copy(ap2, ap);
+        size = vsnprintf(&s, 1, fmt, ap2);
+        va_end(ap2);
+        *strp = malloc(size + 1);
+        if (!*strp)
+                return -1;
+        vsnprintf(*strp, size + 1, fmt, ap);
+
+        return size;
+}
+#endif
+
 /**
  * kit_strdup_printf:
  * @format: sprintf(3) format string
@@ -833,9 +875,9 @@ kit_string_ensure_size (KitString *s, size_t new_size)
                 if (p == NULL)
                         goto oom;
                 /* zero the new block we got */
-                memset (s->buf + s->buf_len, 0, grow_to - s->buf_len);
                 s->buf = p;
-                s->buf_len += KIT_STRING_BLOCK_SIZE;
+                memset (s->buf + s->buf_len, 0, grow_to - s->buf_len);
+                s->buf_len = grow_to;
         }
 
         return TRUE;
