@@ -49,6 +49,7 @@
 #include <grp.h>
 #include <unistd.h>
 #include <errno.h>
+#include <limits.h>
 
 #include <polkit/polkit-private.h>
 #include "polkit-simple.h"
@@ -372,7 +373,7 @@ polkit_auth_obtain (const char *action_id, polkit_uint32_t xid, pid_t pid, DBusE
                                   DBUS_TYPE_UINT32, &xid, 
                                   DBUS_TYPE_UINT32, &pid,
                                   DBUS_TYPE_INVALID);
-	reply = dbus_connection_send_with_reply_and_block (bus, message, -1, error);
+	reply = dbus_connection_send_with_reply_and_block (bus, message, INT_MAX, error);
 	if (reply == NULL || dbus_error_is_set (error)) {
                 ret = _auth_show_dialog_text (action_id, pid, error);
 		goto out;
@@ -397,11 +398,196 @@ out:
 }
 
 
+/**
+ * polkit_dbus_error_generate:
+ * @action: the action that the caller needs an authorization for
+ * @result: the result from e.g. polkit_context_is_caller_authorized()
+ * @error: the #DBusError to set
+ *
+ * Convenience function to generate a #DBusError that encapsulates
+ * information that the caller is not authorized. This includes
+ * information such as @action that describes what action the caller
+ * lacks an authorization for, as well as @result that describes if
+ * the caller can obtain an authorization through authentication.
+ *
+ * Typically a privileged mechanism uses this function to generate
+ * errors. At the other end of the wire, the caller can use
+ * polkit_dbus_error_parse() to extract @action and @result.
+ *
+ * The form of the #DBusError is as follows. The name is
+ * set to
+ * <literal>org.freedesktop.PolicyKit.Error.NotAuthorized</literal>
+ * and the message consists of two strings separated by a single
+ * space: the string representation of the action
+ * (cf. polkit_action_to_string_representation()) and the string
+ * representation of the result
+ * (cf. polkit_result_to_string_representation()).
+ *
+ * This function is in <literal>libpolkit-dbus</literal>.
+ *
+ * Returns: TRUE if @error was set. FALSE on error or OOM.
+ *
+ * Since: 0.8
+ */
+polkit_bool_t
+polkit_dbus_error_generate (PolKitAction *action, PolKitResult result, DBusError *error)
+{
+        polkit_bool_t ret;
+        const char *action_str;
+        const char *result_str;
+
+        ret = FALSE;
+
+        kit_return_val_if_fail (error != NULL && !dbus_error_is_set (error), FALSE);
+        kit_return_val_if_fail (action != NULL && polkit_action_validate (action), FALSE);
+
+        action_str = polkit_action_to_string_representation (action);
+        if (action_str == NULL)
+                goto out;
+
+        result_str = polkit_result_to_string_representation (result);
+        if (result_str == NULL)
+                goto out;
+
+        dbus_set_error (error, 
+                        "org.freedesktop.PolicyKit.Error.NotAuthorized",
+                        "%s %s",
+                        action_str, result_str);
+
+        /* on OOM, error->name and error->message are set to preallocated strings */
+        if (strcmp (error->name, "org.freedesktop.PolicyKit.Error.NotAuthorized") != 0)
+                goto out;
+
+        ret = TRUE;
+
+out:
+        return ret;
+}
+
+/**
+ * polkit_dbus_error_parse:
+ * @error: error to parse; must be set
+ * @action: return location for #PolKitAction object
+ * @result: return location for #PolKitResult variable
+ *
+ * Parse an error received over D-Bus, typically generated by
+ * polkit_dbus_error_generate(), into what action an authorization is
+ * missing for and whether that authorization can be obtained.
+ *
+ * This function is in <literal>libpolkit-dbus</literal>.
+ *
+ * Returns: TRUE only if @error was successfully parsed and @action
+ * and @result is set (and caller must free @action using
+ * polkit_action_unref()).
+ *
+ * Since: 0.8
+ */
+polkit_bool_t
+polkit_dbus_error_parse (DBusError *error, PolKitAction **action, PolKitResult *result)
+{
+        char **tokens;
+        size_t num_tokens;
+        polkit_bool_t ret;
+
+        kit_return_val_if_fail (error != NULL && dbus_error_is_set (error), FALSE);
+        kit_return_val_if_fail (action != NULL, FALSE);
+        kit_return_val_if_fail (result != NULL, FALSE);
+
+        ret = FALSE;
+        tokens = NULL;
+        *action = NULL;
+
+        if (!dbus_error_has_name (error, "org.freedesktop.PolicyKit.Error.NotAuthorized"))
+                goto out;
+
+        tokens = kit_strsplit (error->message, ' ', &num_tokens);
+        if (tokens == NULL || num_tokens != 2)
+                goto out;
+
+        *action = polkit_action_new_from_string_representation (tokens[0]);
+        if (*action == NULL)
+                goto out;
+
+        if (!polkit_result_from_string_representation (tokens[1], result)) {
+                polkit_action_unref (*action);
+                *action = NULL;
+                goto out;
+        }
+        
+        ret = TRUE;
+
+out:
+        if (!ret)
+                *result = POLKIT_RESULT_UNKNOWN;
+
+
+        if (tokens != NULL)
+                kit_strfreev (tokens);
+
+        return ret;
+}
+
+/**
+ * polkit_dbus_error_parse_from_strings:
+ * @error_name: name of D-Bus error
+ * @error_message: message of D-Bus error
+ * @action: return location for #PolKitAction object
+ * @result: return location for #PolKitResult variable
+ *
+ * Like polkit_dbus_error_parse(), only it takes the name and message
+ * instead of a #DBusError. This is useful when usings D-Bus bindings
+ * (such as dbus-glib) that don't expose the #DBusError object
+ * directly.
+ *
+ * This function is in <literal>libpolkit-dbus</literal>.
+ *
+ * Returns: See polkit_dbus_error_parse().
+ *
+ * Since: 0.8
+ */
+polkit_bool_t
+polkit_dbus_error_parse_from_strings (const char *error_name, 
+                                      const char *error_message, 
+                                      PolKitAction **action, 
+                                      PolKitResult *result)
+{
+        DBusError error;
+
+        dbus_error_init (&error);
+        dbus_set_error_const (&error, error_name, error_message);
+
+        return polkit_dbus_error_parse (&error, action, result);
+}
+
 #ifdef POLKIT_BUILD_TESTS
 
 static polkit_bool_t
 _run_test (void)
 {
+        PolKitAction *a;
+        PolKitResult r;
+
+        a = polkit_action_new ();
+        r = POLKIT_RESULT_ONLY_VIA_SELF_AUTH;
+        if (a != NULL) {
+                if (polkit_action_set_action_id (a, "org.example.foo")) {
+                        DBusError error;
+
+                        dbus_error_init (&error);
+                        if (polkit_dbus_error_generate (a, r, &error)) {
+                                PolKitAction *a2;
+                                PolKitResult r2;
+
+                                if (polkit_dbus_error_parse_from_strings (error.name, error.message, &a2, &r2)) {
+                                        kit_assert (polkit_action_equal (a, a2));
+                                        kit_assert (r == r2);
+                                        polkit_action_unref (a2);
+                                }
+                        }
+                }
+                polkit_action_unref (a);
+        }
+
         return TRUE;
 }
 
