@@ -35,9 +35,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <sys/wait.h>
+#include <signal.h>
 
 #include <glib.h>
 #include "polkit-grant.h"
@@ -301,7 +303,7 @@ polkit_grant_io_func (PolKitGrant *polkit_grant, int fd)
         g_return_if_fail (polkit_grant != NULL);
         g_return_if_fail (polkit_grant->helper_is_running);
 
-        while (getline (&line, &line_len, polkit_grant->child_stdout_f) != -1) {
+        while (kit_getline (&line, &line_len, polkit_grant->child_stdout_f) != -1) {
                 if (strlen (line) > 0 &&
                     line[strlen (line) - 1] == '\n')
                         line[strlen (line) - 1] = '\0';
@@ -418,6 +420,8 @@ polkit_grant_io_func (PolKitGrant *polkit_grant, int fd)
 
         if (line != NULL)
                 free (line);
+
+        polkit_grant->func_remove_watch (polkit_grant, polkit_grant->io_watch_id);
 }
 
 /**
